import { useQuery } from '@apollo/client'
import { ChipAction } from 'holi-bricks/components/chips'
import React, { memo, useMemo } from 'react'
import { useTranslation } from 'react-i18next'
import { Platform, StyleProp, StyleSheet, View, ViewStyle, useWindowDimensions } from 'react-native'

import { Skill, Topic } from '@holi/core/domain/shared/types'
import { useErrorHandling } from '@holi/core/errors/hooks'
import { useGeolocationDetails } from '@holi/core/geolocation/hooks/useGeolocationDetails'
import { isSSR } from '@holi/core/helpers/isSSR'
import useRouting from '@holi/core/navigation/hooks/useRouting'
import SpacesHorizontalSlider from '@holi/core/screens/spaces/discover/components/SpacesHorizontalSlider'
import SpacesSeeMore from '@holi/core/screens/spaces/discover/components/SpacesSeeMore'
import {
  DiscoverSpacesData,
  DiscoverSpacesVariables,
  discoverSpacesQuery,
} from '@holi/core/screens/spaces/discover/queries'
import { Space, SpaceMember } from '@holi/core/screens/spaces/types'
import { MapPinLine } from '@holi/icons/src/generated'
import HoliBlockItem from '@holi/ui/components/molecules/HoliBlockItem'
import { PropsWithSkeleton } from '@holi/ui/components/types'
import { HoliTheme, useTheme } from '@holi/ui/styles/theme'

interface BaseProps {
  style?: StyleProp<ViewStyle>
  testID?: string
  pageSize: number
}

interface SelfLoadingTopicProps extends BaseProps {
  topic: Topic
  skill?: never
  locationId?: never
}

interface SelfLoadingSkillProps extends BaseProps {
  topic?: never
  skill: Skill
  locationId?: never
}

interface SelfLoadingLocationProps extends BaseProps {
  topic?: never
  skill?: never
  locationId?: string
}

type ContentProps = SelfLoadingTopicProps | SelfLoadingSkillProps | SelfLoadingLocationProps

const SpacesGroupSlider = ({
  loading: manualLoading = false,
  style,
  testID = 'spaces-group-slider',
  topic,
  skill,
  locationId,
  pageSize = 5,
}: PropsWithSkeleton<ContentProps>) => {
  const { t } = useTranslation()
  const { displayError } = useErrorHandling()
  const { navigate } = useRouting()
  const { width: windowWidth } = useWindowDimensions()
  const { theme } = useTheme()
  const styles = useMemo(() => getStyles(theme), [theme])

  const { geolocationDetails, loadingGeolocation } = useGeolocationDetails(locationId)

  // cardWidth is used for image scaling (see usage). this is fine,
  // because value is constant in web and for mobile windowWidth doesn't change
  const cardWidth = Platform.OS === 'web' ? 320 : windowWidth / 1.5 - 12

  const topicFilter = topic ? { topicIds: [topic.id] } : {}
  const skillFilter = skill ? { skillIds: [skill.id] } : {}
  const { data } = useQuery<DiscoverSpacesData, DiscoverSpacesVariables>(discoverSpacesQuery, {
    variables: {
      // component is either parameterized with topic or skill, so this doesn't fetch both
      ...topicFilter,
      ...skillFilter,
      geolocation: geolocationDetails?.geolocation,
      limit: pageSize,
      offset: 0,
    },
    skip: manualLoading || loadingGeolocation,
    fetchPolicy: isSSR ? 'cache-first' : 'cache-and-network',
    onError: (error) =>
      displayError(error, 'Failed to discover spaces', {
        displayMessage: t('spaces.spaceListing.error'),
        location: 'SpacesGroupSlider.discoverSpacesQuery',
      }),
  })
  const fetchedSpaces = data?.discoverSpaces?.[0]?.spaces

  const { totalResults = 0, data: spacesData = [] } = fetchedSpaces ?? {}
  const hasMore = totalResults > (pageSize ?? 0) // false in loading state

  const topicQuery = topic ? 'topic=' + topic.slug : ''
  const skillQuery = skill ? 'skill=' + skill.slug : ''
  const locationQuery = locationId ? 'location=' + locationId : ''

  const seeMoreQueryParams = [topicQuery, skillQuery, locationQuery].filter((query) => !!query).join('&')

  const loading = manualLoading || !fetchedSpaces || loadingGeolocation

  if (!loading && totalResults === 0) {
    // some topics or skills don't have associated spaces, then we just choose to not display them for now
    return <></>
  }

  return (
    <View style={[styles.sliderContainer, style]} testID={testID}>
      <View style={styles.sliderHeader}>
        {topic && (
          <ChipAction onPress={() => navigate(`/spaces/filter?topic=${topic.slug}`)} color="green">
            {topic.title}
          </ChipAction>
        )}
        {!!locationId && (
          <ChipAction onPress={() => navigate(`/spaces/filter?location=${locationId}`)} iconLeading={MapPinLine}>
            {geolocationDetails?.name ?? ''}
          </ChipAction>
        )}
        {skill && (
          <ChipAction onPress={() => navigate(`/spaces/filter?skill=${skill.slug}`)} color="blue">
            {skill.title}
          </ChipAction>
        )}
      </View>
      <SpacesHorizontalSlider
        loading={loading}
        testID="spaces-group-slider"
        cardWidth={cardWidth}
        slideGap={8}
        seeMoreButton={
          hasMore ? (
            <SpacesSeeMore
              style={styles.seeMoreCard}
              onPress={() => navigate(`/spaces/filter?${seeMoreQueryParams}`)}
              testID={`${testID}-see-more`}
            />
          ) : undefined
        }
      >
        {loading
          ? new Array(pageSize).fill(1).map((_, idx) => {
              return <HoliBlockItem key={idx} loading={true} size="large" testID="space-link" />
            })
          : spacesData.map((space: Space) => (
              <HoliBlockItem
                key={space.id}
                label={space.title}
                title={space.title}
                subtitle={space.location}
                description={space.description}
                imageSrc={space.cover}
                imageBlurhash={space.coverBlurhash}
                users={space.members.data.map((membership: SpaceMember) => membership.user)}
                href={`/spaces/${space.name}`}
                size="large"
                testID="space-link"
                totalUserCount={space.members.totalResults}
                cardWidthForImageScaling={cardWidth}
              />
            ))}
      </SpacesHorizontalSlider>
    </View>
  )
}
export default memo(SpacesGroupSlider)

const getStyles = (theme: HoliTheme) =>
  StyleSheet.create({
    sliderContainer: {
      marginBottom: 24,
    },
    sliderHeader: {
      flexDirection: 'row',
      alignItems: 'center',
    },
    seeMoreCard: {
      paddingVertical: 8,
    },
    spaceCard: {
      paddingVertical: 8,
      paddingHorizontal: 8,
      marginRight: 8,
    },
    backgroundWhite: {
      backgroundColor: theme.colors.background10,
    },
  })
